<?php
/**
 * A general file class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseFile extends BaseFrameworkSystem {
	/**
	 * Separator for header data
	 */
	const SEPARATOR_HEADER_DATA = 0x01;

	/**
	 * Separator header->entries
	 */
	const SEPARATOR_HEADER_ENTRIES = 0x02;

	/**
	 * Separator hash->name
	 */
	const SEPARATOR_HASH_NAME = 0x03;

	/**
	 * Separator entry->entry
	 */
	const SEPARATOR_ENTRIES = 0x04;

	/**
	 * Separator type->position
	 */
	const SEPARATOR_TYPE_POSITION = 0x05;

	/**
	 * Length of count
	 */
	const LENGTH_COUNT = 20;

	/**
	 * Length of position
	 */
	const LENGTH_POSITION = 20;

	/**
	 * Length of name
	 */
	const LENGTH_NAME = 10;

	/**
	 * Maximum length of entry type
	 */
	const LENGTH_TYPE = 20;

	//***** Array elements for 'gaps' array *****

	/**
	 * Start of gap
	 */
	const GAPS_INDEX_START = 'start';

	/**
	 * End of gap
	 */
	const GAPS_INDEX_END = 'end';

	/**
	 * Length of output from hash()
	 */
	private static $hashLength = NULL;

	/**
	 * Counter for total entries
	 */
	private $totalEntries = 0;

	/**
	 * Current seek position
	 */
	private $seekPosition = 0;

	/**
	 * Size of header
	 */
	private $headerSize = 0;

	/**
	 * File header
	 */
	private $header = array();

	/**
	 * Seek positions for gaps ("fragmentation")
	 */
	private $gaps = array();

	/**
	 * Seek positions for damaged entries (e.g. mismatching hash sum, ...)
	 */
	private $damagedEntries = array();

	/**
	 * The current file we are working in
	 */
	private $fileName = '';

	/**
	 * Back-buffer
	 */
	private $backBuffer = '';

	/**
	 * Currently loaded block (will be returned by current())
	 */
	private $currentBlock = '';

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
y	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Init counters and gaps array
		$this->initCountersGapsArray();
	}

	/**
	 * Destructor for cleaning purposes, etc
	 *
	 * @return	void
	 */
	public final function __destruct() {
		// Try to close a file
		$this->closeFile();

		// Call the parent destructor
		parent::__destruct();
	}

	/**
	 * Initializes counter for valid entries, arrays for damaged entries and
	 * an array for gap seek positions. If you call this method on your own,
	 * please re-analyze the file structure. So you are better to call
	 * analyzeFile() instead of this method.
	 *
	 * @return	void
	 */
	protected function initCountersGapsArray () {
		// Init counter and seek position
		$this->setCounter(0);
		$this->setSeekPosition(0);

		// Init arrays
		$this->gaps = array();
		$this->damagedEntries = array();
	}

	/**
	 * Getter for total entries
	 *
	 * @return	$totalEntries	Total entries in this file
	 */
	protected final function getCounter () {
		// Get it
		return $this->totalEntries;
	}

	/**
	 * Setter for total entries
	 *
	 * @param	$totalEntries	Total entries in this file
	 * @return	void
	 */
	protected final function setCounter ($counter) {
		// Set it
		$this->totalEntries = $counter;
	}

	/**
	 * Increment counter
	 *
	 * @return	void
	 */
	protected final function incrementCounter () {
		// Get it
		$this->totalEntries++;
	}

	/**
	 * Getter for header size
	 *
	 * @return	$totalEntries	Size of file header
	 */
	public final function getHeaderSize () {
		// Get it
		return $this->headerSize;
	}

	/**
	 * Setter for header size
	 *
	 * @param	$headerSize		Size of file header
	 * @return	void
	 */
	protected final function setHeaderSize ($headerSize) {
		// Set it
		$this->headerSize = $headerSize;
	}

	/**
	 * Getter for header array
	 *
	 * @return	$totalEntries	Size of file header
	 */
	protected final function getHeade () {
		// Get it
		return $this->header;
	}

	/**
	 * Setter for header
	 *
	 * @param	$header		Array for a file header
	 * @return	void
	 */
	protected final function setHeader (array $header) {
		// Set it
		$this->header = $header;
	}

	/**
	 * Getter for seek position
	 *
	 * @return	$seekPosition	Current seek position (stored here in object)
	 */
	protected final function getSeekPosition () {
		// Get it
		return $this->seekPosition;
	}

	/**
	 * Setter for seek position
	 *
	 * @param	$seekPosition	Current seek position (stored here in object)
	 * @return	void
	 */
	protected final function setSeekPosition ($seekPosition) {
		// And set it
		$this->seekPosition = $seekPosition;
	}

	/**
	 * Updates seekPosition attribute from file to avoid to much access on file.
	 *
	 * @return	void
	 */
	protected function updateSeekPosition () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Get key (= seek position)
		$seekPosition = $this->getIteratorInstance()->key();
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Setting seekPosition=%s', __METHOD__, __LINE__, $seekPosition));

		// And set it here
		$this->setSeekPosition($seekPosition);

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Seeks to beginning of file, updates seek position in this object and
	 * flushes the header.
	 *
	 * @return	void
	 */
	protected function rewindUpdateSeekPosition () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// flushFileHeader must be callable
		assert(is_callable(array($this, 'flushFileHeader')));

		// Seek to beginning of file
		$this->getIteratorInstance()->rewind();

		// And update seek position ...
		$this->updateSeekPosition();

		// ... to write it back into the file
		$this->flushFileHeader();

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!!', __METHOD__, __LINE__));
	}

	/**
	 * Seeks to old position
	 *
	 * @return	void
	 */
	protected function seekToOldPosition () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Seek to currently ("old") saved position
		$this->getIteratorInstance()->seek($this->getSeekPosition());

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!!', __METHOD__, __LINE__));
	}

	/**
	 * Checks whether the block separator has been found
	 *
	 * @param	$str		String to look in
	 * @return	$isFound	Whether the block separator has been found
	 */
	public static function isBlockSeparatorFound ($str) {
		// Determine it
		$isFound = (strpos($str, chr(self::SEPARATOR_ENTRIES)) !== FALSE);

		// Return result
		return $isFound;
	}

	/**
	 * Getter for the file pointer
	 *
	 * @return	$filePointer	The file pointer which shall be a valid file resource
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public final function getPointer () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Setter for file name
	 *
	 * @param	$fileName	The new file name
	 * @return	void
	 */
	protected final function setFileName ($fileName) {
		$fileName = (string) $fileName;
		$this->fileName = $fileName;
	}

	/**
	 * Getter for file name
	 *
	 * @return	$fileName	The current file name
	 */
	public final function getFileName () {
		return $this->fileName;
	}

	/**
	 * Initializes the back-buffer by setting it to an empty string.
	 *
	 * @return	void
	 */
	private function initBackBuffer () {
		// Simply call the setter
		$this->setBackBuffer('');
	}

	/**
	 * Setter for backBuffer field
	 *
	 * @param	$backBuffer		Characters to "store" in back-buffer
	 * @return	void
	 */
	private function setBackBuffer ($backBuffer) {
		// Cast to string (so no arrays or objects)
		$backBuffer = (string) $backBuffer;

		// ... and set it
		$this->backBuffer = $backBuffer;
	}

	/**
	 * Getter for backBuffer field
	 *
	 * @return	$backBuffer		Characters "stored" in back-buffer
	 */
	private function getBackBuffer () {
		return $this->backBuffer;
	}

	/**
	 * Setter for currentBlock field
	 *
	 * @param	$currentBlock		Characters to set a currently loaded block
	 * @return	void
	 */
	private function setCurrentBlock ($currentBlock) {
		// Cast to string (so no arrays or objects)
		$currentBlock = (string) $currentBlock;

		// ... and set it
		$this->currentBlock = $currentBlock;
	}

	/**
	 * Gets currently read data
	 *
	 * @return	$current	Currently read data
	 */
	public function getCurrentBlock () {
		// Return it
		return $this->currentBlock;
	}

	/**
	 * Initializes this file class
	 *
	 * @param	$fileName	Name of this abstract file
	 * @return	void
	 */
	protected function initFile ($fileName) {
		// Get a file i/o pointer instance
		$pointerInstance = ObjectFactory::createObjectByConfiguredName('file_raw_input_output_class', array($fileName));

		// ... and set it here
		$this->setPointerInstance($pointerInstance);
	}

	/**
	 * Writes data at given position
	 *
	 * @param	$seekPosition	Seek position
	 * @param	$data			Data to be written
	 * @param	$flushHeader	Whether to flush the header (default: flush)
	 * @return	void
	 */
	protected function writeData ($seekPosition, $data, $flushHeader = TRUE) {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] seekPosition=%s,data()=%s - CALLED!', __METHOD__, __LINE__, $seekPosition, strlen($data)));

		// Write data at given position
		$this->getPointerInstance()->writeAtPosition($seekPosition, $data);

		// Update seek position
		$this->updateSeekPosition();

		// Flush the header?
		if ($flushHeader === TRUE) {
			// Flush header
			$this->flushFileHeader();

			// Seek to old position
			$this->seekToOldPosition();
		} // END - if

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Marks the currently loaded block as empty (with length of the block)
	 *
	 * @param	$length		Length of the block
	 * @return	void
	 */
	protected function markCurrentBlockAsEmpty ($length) {
		// Get current seek position
		$currentPosition = $this->key();

		// Now add it as gap entry
		array_push($this->gaps, array(
			self::GAPS_INDEX_START  => ($currentPosition - $length),
			self::GAPS_INDEX_END    => $currentPosition,
		));
	}

	/**
	 * Checks whether the file header is initialized
	 *
	 * @return	$isInitialized	Whether the file header is initialized
	 */
	public function isFileHeaderInitialized () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Default is not initialized
		$isInitialized = FALSE;

		// Is the file initialized?
		if ($this->isFileInitialized()) {
			// Some bytes has been written, so rewind to start of it.
			$rewindStatus = $this->rewind();
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] rewindStatus=%s', __METHOD__, __LINE__, $rewindStatus));

			// Is the rewind() call successfull?
			if ($rewindStatus != 1) {
				// Something bad happened
				self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Could not rewind().', __METHOD__, __LINE__));
			} // END - if

			// Read file header
			$this->readFileHeader();

			// The above method does already check the header
			$isInitialized = TRUE;
		} // END - if

		// Return result
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] isInitialized=%d - EXIT!', __METHOD__, __LINE__, intval($isInitialized)));
		return $isInitialized;
	}

	/**
	 * Checks whether the assigned file has been initialized
	 *
	 * @return	$isInitialized		Whether the file's size is zero
	 */
	public function isFileInitialized () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Get it from iterator which holds the pointer instance. If FALSE is returned
		$fileSize = $this->size();
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] fileSize=%s', __METHOD__, __LINE__, $fileSize));

		/*
		 * The returned file size should not be FALSE or NULL as this means
		 * that the pointer class does not work correctly.
		 */
		assert(is_int($fileSize));

		// Is more than 0 returned?
		$isInitialized = ($fileSize > 0);

		// Return result
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] isInitialized=%d - EXIT!', __METHOD__, __LINE__, intval($isInitialized)));
		return $isInitialized;
	}

	/**
	 * Creates the assigned file
	 *
	 * @return	void
	 */
	public function createFileHeader () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// The file's header should not be initialized here
		assert(!$this->isFileHeaderInitialized());

		// Simple flush file header which will create it.
		$this->flushFileHeader();

		// Rewind seek position (to beginning of file) and update/flush file header
		$this->rewindUpdateSeekPosition();

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!!', __METHOD__, __LINE__));
	}

	/**
	 * Pre-allocates file (if enabled) with some space for later faster write access.
	 *
	 * @param	$type	Type of the file
	 * @return	void
	 */
	public function preAllocateFile ($type) {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Is it enabled?
		if ($this->getConfigInstance()->getConfigEntry($type . '_pre_allocate_enabled') != 'Y') {
			// Not enabled
			self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Not pre-allocating file.', __METHOD__, __LINE__));

			// Don't continue here.
			return;
		} // END - if

		// Message to user
		self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Pre-allocating file ...', __METHOD__, __LINE__));

		// Calculate minimum length for one entry
		$minLengthEntry = $this->getBlockInstance()->calculateMinimumBlockLength();
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] minLengthEntry=%s', __METHOD__, __LINE__, $minLengthEntry));

		// Calulcate seek position
		$seekPosition = $minLengthEntry * $this->getConfigInstance()->getConfigEntry($type . '_pre_allocate_count');
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] seekPosition=%s', __METHOD__, __LINE__, $seekPosition));

		// Now simply write a NUL there. This will pre-allocate the file.
		$this->writeData($seekPosition, chr(0));

		// Rewind seek position (to beginning of file) and update/flush file header
		$this->rewindUpdateSeekPosition();

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Close a file source and set it's instance to null and the file name
	 * to empty
	 *
	 * @return	void
	 * @todo	~10% done?
	 */
	public function closeFile () {
		$this->partialStub('Unfinished method.');

		// Remove file name
		$this->setFileName('');
	}

	/**
	 * Determines seek position
	 *
	 * @return	$seekPosition	Current seek position
	 */
	public function determineSeekPosition () {
		// Call pointer instance
		return $this->getPointerInstance()->determineSeekPosition();
	}

	/**
	 * Seek to given offset (default) or other possibilities as fseek() gives.
	 *
	 * @param	$offset		Offset to seek to (or used as "base" for other seeks)
	 * @param	$whence		Added to offset (default: only use offset to seek to)
	 * @return	$status		Status of file seek: 0 = success, -1 = failed
	 */
	public function seek ($offset, $whence = SEEK_SET) {
		// Call pointer instance
		return $this->getPointerInstance()->seek($offset, $whence);
	}

	/**
	 * Size of this file
	 *
	 * @return	$size	Size (in bytes) of file
	 * @todo	Handle seekStatus
	 */
	public function size () {
		// Call pointer instance
		return $this->getPointerInstance()->size();
	}

	/**
	 * Read data a file pointer
	 *
	 * @return	mixed	The result of fread()
	 * @throws	NullPointerException	If the file pointer instance
	 *									is not set by setPointer()
	 * @throws	InvalidResourceException	If there is being set
	 */
	public function readFromFile () {
		// Call pointer instance
		return $this->getPointerInstance()->readFromFile();
	}

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read
	 * @return	$data	Data read from file
	 */
	public function read ($bytes) {
		// Call pointer instance
		return $this->getPointerInstance()->read($bytes);
	}

	/**
	 * Write data to a file pointer
	 *
	 * @param	$dataStream		The data stream we shall write to the file
	 * @return	mixed			Number of writes bytes or FALSE on error
	 * @throws	NullPointerException	If the file pointer instance
	 *									is not set by setPointer()
	 * @throws	InvalidResourceException	If there is being set
	 *											an invalid file resource
	 */
	public function writeToFile ($dataStream) {
		// Call pointer instance
		return $this->getPointerInstance()->writeToFile($dataStream);
	}

	/**
	 * Rewinds to the beginning of the file
	 *
	 * @return	$status 	Status of this operation
	 */
	public function rewind () {
		// Call pointer instance
		return $this->getPointerInstance()->rewind();
	}

	/**
	 * Determines whether the EOF has been reached
	 *
	 * @return	$isEndOfFileReached		Whether the EOF has been reached
	 */
	public final function isEndOfFileReached () {
		// Call pointer instance
		return $this->getPointerInstance()->isEndOfFileReached();
	}

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 */
	public function analyzeFile () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Make sure the file is initialized
		assert($this->isFileInitialized());

		// Init counters and gaps array
		$this->initCountersGapsArray();

		// Output message (as this may take some time)
		self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Analyzing file structure ... (this may take some time)', __METHOD__, __LINE__));

		// First rewind to the begining
		$this->rewind();

		// Then try to load all entries
		while ($this->valid()) {
			// Go to next entry
			$this->next();

			// Get current entry
			$current = $this->getCurrentBlock();

			// Debug message
			/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] current()=%s', __METHOD__, __LINE__, strlen($current)));
		} // END - while

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Advances to next "block" of bytes
	 *
	 * @return	void
	 */
	public function next () {
		// Is there nothing to read?
		if (!$this->valid()) {
			// Nothing to read
			return;
		} // END - if

		// Make sure the block instance is set
		assert($this->getBlockInstance() instanceof CalculatableBlock);

		// First calculate minimum block length
		$length = $this->getBlockInstance()->calculateMinimumBlockLength();

		// Short be more than zero!
		assert($length > 0);

		// Read possibly back-buffered bytes from previous call of next().
		$data = $this->getBackBuffer();

		/*
		 * Read until a entry/block separator has been found. The next read
		 * "block" may not fit, so this loop will continue until the EOB or EOF
		 * has been reached whatever comes first.
		 */
		while ((!$this->isEndOfFileReached()) && (!self::isBlockSeparatorFound($data))) {
			// Then read the next possible block
			$block = $this->read($length);

			// Is it all empty?
			if (strlen(trim($block)) == 0) {
				// Mark this block as empty
				$this->markCurrentBlockAsEmpty($length);

				// Skip to next block
				continue;
			} // END - if

			// At this block then
			$data .= $block;

			// A debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d] data()=%s', __FUNCTION__, __LINE__, strlen($data)));
		} // END - if

		// EOF reached?
		if ($this->isEndOfFileReached()) {
			// Set whole data as current read block
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('Calling setCurrentBlock(' . strlen($data) . ') ...');
			$this->setCurrentBlock($data);

			// Then abort here silently
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('EOF reached.');
			return;
		} // END - if

		/*
		 * Init back-buffer which is the data that has been found beyond the
		 * separator.
		 */
		$this->initBackBuffer();

		// Separate data
		$dataArray = explode(chr(self::SEPARATOR_ENTRIES), $data);

		// This array must contain two elements
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('dataArray=' . print_r($dataArray, TRUE));
		assert(count($dataArray) == 2);

		// Left part is the actual block, right one the back-buffer data
		$this->setCurrentBlock($dataArray[0]);
		$this->setBackBuffer($dataArray[1]);
	}

	/**
	 * Checks wether the current entry is valid (not at the end of the file).
	 * This method will return TRUE if an emptied (nulled) entry has been found.
	 *
	 * @return	$isValid	Whether the next entry is valid
	 */
	public function valid () {
		// Make sure the block instance is set
		assert($this->getBlockInstance() instanceof Block);

		// First calculate minimum block length
		$length = $this->getBlockInstance()->calculateMinimumBlockLength();

		// Short be more than zero!
		assert($length > 0);

		// Get current seek position
		$seekPosition = $this->key();

		// Then try to read it
		$data = $this->read($length);

		// If some bytes could be read, all is fine
		$isValid = ((is_string($data)) && (strlen($data) > 0));

		// Get header size
		$headerSize = $this->getBlockInstance()->getHeaderSize();

		// Is the seek position at or beyond the header?
		if ($seekPosition >= $headerSize) {
			// Seek back to old position
			$this->seek($seekPosition);
		} else {
			// Seek directly behind the header
			$this->seek($headerSize);
		}

		// Return result
		return $isValid;
	}

	/**
	 * Gets current seek position ("key").
	 *
	 * @return	$key	Current key in iteration
	 */
	public function key () {
		// Call pointer instance
		return $this->getPointerInstance()->determineSeekPosition();
	}

	/**
	 * Reads the file header
	 *
	 * @return	void
	 */
	public function readFileHeader () {
		// Make sure the block instance is set
		assert($this->getBlockInstance() instanceof Block);

		// Call block instance
		$this->getBlockInstance()->readFileHeader();
	}

	/**
	 * Flushes the file header
	 *
	 * @return	void
	 */
	public function flushFileHeader () {
		// Make sure the block instance is set
		assert($this->getBlockInstance() instanceof Block);

		// Call block instance
		$this->getBlockInstance()->flushFileHeader();
	}
}

// [EOF]
?>
