<?php
/**
 * A general index class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseIndex extends BaseFrameworkSystem {
	/**
	 * Magic for this index
	 */
	const INDEX_MAGIC = 'INDEXv0.1';

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Calculate header size
		$this->setHeaderSize(
			strlen(self::INDEX_MAGIC) +
			strlen(chr(self::SEPARATOR_HEADER_DATA)) +
			self::LENGTH_COUNT +
			strlen(chr(self::SEPARATOR_HEADER_ENTRIES))
		);

		// Init counters and gaps array
		$this->initCountersGapsArray();
	}

	/**
	 * Reads the file header
	 *
	 * @return	void
	 */
	public function readFileHeader () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// First rewind to beginning as the header sits at the beginning ...
		$this->getIteratorInstance()->rewind();

		// Then read it (see constructor for calculation)
		$data = $this->getIteratorInstance()->read($this->getHeaderSize());
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Read %d bytes (%d wanted).', __METHOD__, __LINE__, strlen($data), $this->getHeaderSize()));

		// Have all requested bytes been read?
		assert(strlen($data) == $this->getHeaderSize());
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Last character must be the separator
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] data(-1)=%s', __METHOD__, __LINE__, dechex(ord(substr($data, -1, 1)))));
		assert(substr($data, -1, 1) == chr(self::SEPARATOR_HEADER_ENTRIES));
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Okay, then remove it
		$data = substr($data, 0, -1);

		// And update seek position
		$this->updateSeekPosition();

		/*
		 * Now split it:
		 *
		 * 0 => magic
		 * 1 => total entries
		 */
		$header = explode(chr(self::SEPARATOR_HEADER_DATA), $data);

		// Set it here
		$this->setHeader($header);

		// Check if the array has only 3 elements
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] header(%d)=%s', __METHOD__, __LINE__, count($header), print_r($header, TRUE)));
		assert(count($header) == 2);
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Check magic
		assert($header[0] == self::INDEX_MAGIC);
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Check length of count
		assert(strlen($header[1]) == self::LENGTH_COUNT);
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Decode count
		$header[1] = hex2bin($header[1]);

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Flushes the file header
	 *
	 * @return	void
	 */
	public function flushFileHeader () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Put all informations together
		$header = sprintf('%s%s%s%s',
			// Magic
			self::INDEX_MAGIC,

			// Separator header data
			chr(self::SEPARATOR_HEADER_DATA),

			// Total entries
			str_pad($this->dec2hex($this->getCounter()), self::LENGTH_COUNT, '0', STR_PAD_LEFT),

			// Separator header<->entries
			chr(self::SEPARATOR_HEADER_ENTRIES)
		);

		// Write it to disk (header is always at seek position 0)
		$this->writeData(0, $header, FALSE);

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Initializes this index
	 *
	 * @param	$fileName	File name of this index
	 * @return	void
	 * @todo	Currently the index file is not cached, please implement a memory-handling class and if enough RAM is found, cache the whole index file.
	 */
	protected function initIndex ($fileName) {
		// Append index file extension
		$fileName .= $this->getConfigInstance()->getConfigEntry('index_extension');

		// Get a file i/o pointer instance for index file
		$fileInstance = ObjectFactory::createObjectByConfiguredName('index_file_class', array($fileName, $this));

		// Get iterator instance
		$iteratorInstance = ObjectFactory::createObjectByConfiguredName('file_iterator_class', array($fileInstance));

		// Is the instance implementing the right interface?
		assert($iteratorInstance instanceof SeekableWritableFileIterator);

		// Set iterator here
		$this->setIteratorInstance($iteratorInstance);

		// Is the file's header initialized?
		if (!$this->getIteratorInstance()->isFileHeaderInitialized()) {
			// No, then create it (which may pre-allocate the index)
			$this->getIteratorInstance()->createFileHeader();

			// And pre-allocate a bit
			$this->getIteratorInstance()->preAllocateFile('index');
		} // END - if

		// Load the file header
		$this->readFileHeader();

		// Count all entries in file
		$this->getIteratorInstance()->analyzeFile();
	}

	/**
	 * Calculates minimum length for one entry/block
	 *
	 * @return	$length		Minimum length for one entry/block
	 */
	public function calculateMinimumBlockLength () {
		// Calulcate it
		$length = self::LENGTH_TYPE + strlen(chr(self::SEPARATOR_TYPE_POSITION)) + self::LENGTH_POSITION + strlen(chr(self::SEPARATOR_ENTRIES));

		// Return it
		return $length;
	}

	/**
	 * Close a file source and set it's instance to null and the file name
	 * to empty.
	 *
	 * @return	void
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function closeFile () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Determines whether the EOF has been reached
	 *
	 * @return	$isEndOfFileReached		Whether the EOF has been reached
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function isEndOfFileReached () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

	/**
	 * Getter for file name
	 *
	 * @return	$fileName	The current file name
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function getFileName () {
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}
}

// [EOF]
?>
